// @(#) bmap.c
// @(#) ~~~~~~
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// @(#) This source is Copyright(c). 1997 by Brent York, All Rights 
// @(#) Reserved.  This source falls under the gnu general public licence. 
// @(#) Mon - 5 - May 11:30Pm - @ - [GCC(Devel for Linux)]
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//  Comment:
//    
//    This source tests graphics.o's functions to make sure they work. 
//  it does this by doing 2d bumpmap.
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// All the headers we need. 
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <vga.h>
#include "graphics.h"
#include "mouse.h"

typedef unsigned char byte;
void envinit(byte *buffer);
void load_pal(char *filename, byte *pal);
void load_pcx(char *filename, byte *buffer);
void bump(int lightx, int lighty, byte *bumpmap, byte *envmap, byte *buffer);

int main(void) {
  byte envmap[256*256];
  byte bumpmap[64000];
  byte pal[768];
  byte *vScr;
  int x,y,b;
 

  if (MOUSE_Init()) {
    printf("No mouse found... exiting.\n");
    return 1;
  }

  vScr=(byte *)malloc(64000);
  memset(vScr,0,64000);

  GFX_Init();
  GFX_Setvidmode(G320x200x256); 
  envinit(envmap);
  load_pal("bmap.pal",pal);
  load_pcx("bmap.pcx",bumpmap);

  GFX_SetPR(pal);
  
  b=0;
  while (!b) {
    MOUSE_Status(&x, &y, &b);
    bump(x, y, bumpmap, envmap, vScr);
    GFX_Vret();
    GFX_Flip(vScr,320,200);
  }

  GFX_Setvidmode(TEXT);
  printf("Copyright(c) 1997 By Nuclear Winter Entertainment Inc, and CScene.\n");
  printf("All Rights Reserved.\n");
  free(vScr);
}



// Inits the envmap.
void envinit(byte *buffer) {
  int y,x;
  float nX, nY, nZ;

  for (y=0;y<256;y++) {
    for (x=0;x<256;x++) {
      nX=(x-128)/128.0;
      nY=(y-128)/128.0;
      nZ=1-sqrt(nX*nX+nY*nY);
      if (nZ<0) nZ=0;
      buffer[x+y*256]=(byte)(255*nZ);
    }
  }
}



// Loads a palette.
void load_pal(char *filename, byte *pal) {
  int i=0;
  FILE *pal_ptr=fopen(filename,"rb");
  
  for (i=0;i<768;i++) 
    pal[i]=fgetc(pal_ptr);
  fclose(pal_ptr);
}



// Loads a 320x200 pcx
void load_pcx(char *filename, byte *buffer) {
  FILE *pcx_ptr=fopen(filename,"rb");
  
  int count=0;
  int cnt=0;
  int chr=0;
  int rpt=0;

  fseek(pcx_ptr,128,SEEK_SET);

  while (count < 64000) {
    chr=fgetc(pcx_ptr);			// Get char
    if ((chr&0xC0)==0xC0) {		// If top two bits are set
      rpt=chr&0x3F;			// Get repeat val
      chr=fgetc(pcx_ptr);		// And next col
      for (cnt=0;cnt<rpt;cnt++) { 	// Repeat it rpt times
        buffer[count]=chr;
        count++;  
      }

    } else {				// Else
      buffer[count]=chr;                // Set the color
      count++;
    }
  }	
  fclose(pcx_ptr);
}



// Do the bumpmapping.
void bump(int lightx, int lighty, byte *bumpmap, byte *envmap, byte *buffer) {
  int i=320;  
  int nX,nY,lX,lY;
  int y,x;

  for (y=1;y<199;y++) {
    for (x=0;x<320;x++) {
      nX=bumpmap[i+1]-bumpmap[i-1];
      nY=bumpmap[i+320]-bumpmap[i-320];
      lX=x-lightx; lY=y-lighty;
      nX-=lX; nY-=lY; nX+=128; nY+=128;
      if (nX<0) nX=0; if (nX>255) nX=255;
      if (nY<0) nY=0; if (nY>255) nY=255;
      buffer[i]=(int)envmap[nX+nY*256];
      i++;
    }
  }
}
